#include "stdafx.h"
#include "clf_random.h"
#include "clf_primitives.h"
#include "clf_geometry.h"
#include "clf_camera.h"
#include "clf_resources.h"

extern HWND  hWnd;						// Window Handle
extern bool keyDown[256];			// holds true / false for each key

static int m_nFPS = 0;
static int m_nFrames = 0;											// FPS and FPS Counter

cCamera g_cam;
clf_stage g_stage;
float anglex=0.f;
float angley=0.f;
float anglez=0.f;
GLuint g_tex_earth = -1;
GLuint g_tex_box = -1;
GLuint g_tex_floor = -1;

struct lst_ob{
	float x,y,z;
	float w,d,h;
	float r,g,b;
};

#define MAX_OBJECTS 750
lst_ob objects[MAX_OBJECTS];
GLuint draw_list = -1;
bool m_use_draw_list = false;

#define __LIGHTS_ENABLED__

// this function is called when the program starts to initialize the game
void game_init()
{
	g_tex_earth = clf_loadTexture("earth2.tga");
	g_tex_box = clf_loadTexture("planks02.tga");
	g_tex_floor = clf_loadTexture("floor.tga");

	g_cam.setup(clf_vector3D(5,50,500),       // position viewer
							clf_vector3D(0,0,0),        // setup the point of interest (where we are looking at)
							clf_vector3D(0,1,0));       // which way is up

	glEnable (GL_DEPTH_TEST);        // enable depth test
	glEnable(GL_COLOR_MATERIAL);     // use color for drawing the object
#ifdef __LIGHTS_ENABLED__
	glEnable (GL_LIGHTING);					 // enable lighting
	glEnable (GL_LIGHT0);						 // and turn on light 0
	static GLfloat light_ambient[] = { 0.0f, 0.0f, 0.0f, 1.f };        // the colour and intesity of the ambient light
	static GLfloat light_diffuse[] = { 0.9f, 0.9f, 0.9f, 1.f };        // the difuse property of the surface
	static GLfloat light_specular[] = { 1.f, 1.f, 1.f, 1.f };    // the specular property
	// pass them to OpenGL
	glLightfv (GL_LIGHT0, GL_AMBIENT, light_ambient);
	glLightfv (GL_LIGHT0, GL_DIFFUSE, light_diffuse);
	glLightfv (GL_LIGHT0, GL_SPECULAR, light_specular);
#endif

	int i;
	init_random(12359);

	for (i=0; i<MAX_OBJECTS; i++)
	{
		objects[i].x = (float)(random(200)-100);
		objects[i].y = (float)0;
		objects[i].z = (float)(random(200)-100);

		objects[i].w = (float)(10+random(10));
		objects[i].d = (float)(10+random(10));
		objects[i].h = (float)(10+random(10));

		objects[i].r = ((float)random(100))/100.f;
		objects[i].g = ((float)random(100))/100.f;
		objects[i].b = ((float)random(100))/100.f;
	}
	clf_disable_textures();
}

// this function is called when the window is resized
void resize_window (int width, int height)			// reshape the window when it's moved or resized
{
	g_stage.resize(0,0,width, height);

	glMatrixMode (GL_PROJECTION);									// select the projection matrix
	glLoadIdentity ();														// reset the projection matrix

	// viewport and perspective
	g_stage.prepare(); 
}

// this function is called upon exit to clear allocated resources
void game_terminate()
{
	// release the textures
	glDeleteTextures(1, &g_tex_earth);
	glDeleteTextures(1, &g_tex_box);
	glDeleteTextures(1, &g_tex_floor);
}

// the core of the game. This is called repeatedly to update the state of the game
// fElapsed: is the elapsed time since last call. The time is in seconds (usualy a very small number)
void frame_move(float fElapsed)
{
	if (keyDown[VK_ESCAPE]) 
		DestroyWindow(hWnd);

	if (keyDown['L']) 
	{
		m_use_draw_list = !m_use_draw_list;
		keyDown['L'] = false;
	}

	anglex += 22.5f * fElapsed; // rotating speed is 22.5 defgrees per second!!
	angley += 22.5f * fElapsed;
	anglez += 22.5f * fElapsed;

	// frame counting mechanism
	// the higher the frame rate the faster the system
	// if frame rate drops below 30 we are in deep trouble
	// we should either optimize the program or buy a new computer
	static dFloat tot = 0;
	tot += fElapsed;
	m_nFrames++;												// Increment Our frame Counter
	if (tot >= 1.f)
	{
		tot = 0;
		m_nFPS = m_nFrames;
		m_nFrames = 0;

		char txt[200];
		sprintf(txt, "GusOnGames, fps:%d, draw list:%s",m_nFPS, m_use_draw_list?"on":"off");
		SetWindowText(hWnd, txt);
	}
}

// the drawing function
// it is called right after the frame_move function
void frame_render()
{
	// clear the screen
	glClearColor (0.f, 0.f, 0.2f, 1.f);				// background colour
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// clear screen and depth buffer

	glMatrixMode (GL_MODELVIEW);								// select the modelview matrix
	glLoadIdentity ();													// reset

	// position the viewer
	g_cam.apply();

	// position the light
#ifdef __LIGHTS_ENABLED__
	static GLfloat light_position[] = { 80.f, 30.f, 0.f, 0.f };				 // the position of the light
	glLightfv (GL_LIGHT0, GL_POSITION, light_position);
#endif


	if (m_use_draw_list)
	{
		if (draw_list == -1)
		{
			draw_list = glGenLists(1);
			glNewList(draw_list, GL_COMPILE);
			// draw a floor
			// set the colour
			glColor3f(1.0f, 1.0f, 1.0f);
			clf_enable_textures(g_tex_floor);
			glBegin(GL_QUADS);
			glNormal3f(0,1,0);    // face normal vector
			glTexCoord2f(0, 0);   // texture map 
			glVertex3f(-250, -2, -250);	 // vertex
			glTexCoord2f(0, 1);
			glVertex3f(-250, -2, 250);
			glTexCoord2f(1, 1);
			glVertex3f(250, -2, 250);
			glTexCoord2f(1, 0);
			glVertex3f(250, -2, -250);
			glEnd();
			int i;
			for (i=0; i<MAX_OBJECTS; i++)
			{
				glPushMatrix();
				glTranslatef(objects[i].x, objects[i].y, objects[i].z);
				glColor3f(objects[i].r, objects[i].g, objects[i].b);
				clf_enable_textures(g_tex_box);
				drawBox(4, 4, 4);
				glPopMatrix();
			}
			glEndList();
		}
		glCallList(draw_list);
	}
	else
	{
		// draw a floor
		// set the colour
		glColor3f(1.0f, 1.0f, 1.0f);
		clf_enable_textures(g_tex_floor);
		glBegin(GL_QUADS);
		glNormal3f(0,1,0);    // face normal vector
		glTexCoord2f(0, 0);   // texture map 
		glVertex3f(-250, -2, -250);	 // vertex
		glTexCoord2f(0, 1);
		glVertex3f(-250, -2, 250);
		glTexCoord2f(1, 1);
		glVertex3f(250, -2, 250);
		glTexCoord2f(1, 0);
		glVertex3f(250, -2, -250);
		glEnd();
		int i;
		for (i=0; i<MAX_OBJECTS; i++)
		{
			glPushMatrix();
			glTranslatef(objects[i].x, objects[i].y, objects[i].z);
			glColor3f(objects[i].r, objects[i].g, objects[i].b);
			clf_enable_textures(g_tex_box);
			drawBox(4, 4, 4);
			glPopMatrix();
		}
	}
}
